package cn.iocoder.yudao.module.infra.controller.admin.file;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.servlet.ServletUtils;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FilePageReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FileRespVO;
import cn.iocoder.yudao.module.infra.convert.file.FileConvert;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ClassUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.FILE_IS_EMPTY;

@Api(tags = "管理后台 - 文件存储")
@RestController
@RequestMapping("/infra/file")
@Validated
@Slf4j
public class FileController {

    @Value("${ue.server-url}")
    private String path;

    @Value("${ue.root-path}")
    private String prePath;

    @Resource
    private FileService fileService;

    @PostMapping("upload")
    @ApiOperation("上传文件")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "query", dataType="file", name = "file", value = "文件附件", required = true, dataTypeClass = MultipartFile.class),
        @ApiImplicitParam(name = "path", value = "文件路径", example = "yudaoyuanma.png", dataTypeClass = String.class)
    })
    public CommonResult<String> uploadFile(@RequestPart("file") MultipartFile file,
                                           @RequestParam(value = "path", required = false) String path) throws Exception {
        if (StrUtil.isBlank(path)) {
            String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
            path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
        }
        return success(fileService.createFile(path,
                IoUtil.readBytes(file.getInputStream())));
    }

    @PostMapping("org-name/up")
    @ApiOperation("上传文件(源文件名称返回)")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType="file", name = "file", value = "文件附件", required = true, dataTypeClass = MultipartFile.class)
    })
    public CommonResult<String> upFile(@RequestPart("file") MultipartFile file) throws Exception {
            String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
            String path = DateUtils.getDir() + FileNameUtil.getPrefix(file.getOriginalFilename()) + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
        return success(fileService.createFile(path, IoUtil.readBytes(file.getInputStream())));
    }

    @PostMapping("up")
    @ApiOperation("文件上传")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "query", dataType="file", name = "file", value = "文件附件", required = true, dataTypeClass = MultipartFile.class)
    })
    public CommonResult<String> up(@RequestPart("file") MultipartFile file) throws Exception {
        if (Objects.isNull(file)) { throw exception(FILE_IS_EMPTY); }
        // 获取文件后缀
        String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
        String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
        return success(fileService.createFile(path, file.getBytes()));
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除文件")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('infra:file:delete')")
    public CommonResult<Boolean> deleteFile(@RequestParam("id") Long id) throws Exception {
        fileService.deleteFile(id);
        return success(true);
    }

    @GetMapping("/{configId}/get/{path}")
    @ApiOperation("下载文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "configId", value = "配置编号",  required = true, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "path", value = "文件路径", required = true, dataTypeClass = String.class)
    })
    public void getFileContent(HttpServletResponse response,
                               @PathVariable("configId") Long configId,
                               @PathVariable("path") String path) throws Exception {
        byte[] content = fileService.getFileContent(configId, path);
        if (content == null) {
            log.warn("[getFileContent][configId({}) path({}) 文件不存在]", configId, path);
            response.setStatus(HttpStatus.NOT_FOUND.value());
            return;
        }
        ServletUtils.writeAttachment(response, path, content);
    }

    @GetMapping("/page")
    @ApiOperation("获得文件分页")
    @PreAuthorize("@ss.hasPermission('infra:file:query')")
    public CommonResult<PageResult<FileRespVO>> getFilePage(@Valid FilePageReqVO pageVO) {
        PageResult<FileDO> pageResult = fileService.getFilePage(pageVO);
        return success(FileConvert.INSTANCE.convertPage(pageResult));
    }

    @RequestMapping("/editor")
    public void editor(HttpServletRequest request, HttpServletResponse response,
                                 @RequestParam(value = "action") String action, @RequestParam(value = "upfile",required = false) MultipartFile upfile) throws IOException {
        InputStreamReader reader = null;
        BufferedReader bfReader = null;
        String retStr = "errorConfig";
        OutputStream outputStream = null;
        try {
            outputStream = response.getOutputStream();
            if ("config".equals(action)) {
                response.setCharacterEncoding("utf-8");
                response.setContentType("application/json");
                String realPath = prePath+path.replace("controller","")+"config.json";
                StringBuilder builder = new StringBuilder();
                reader = new InputStreamReader(new FileInputStream(realPath), "UTF-8");
                bfReader = new BufferedReader(reader);
                String tmpContent = null;
                while ((tmpContent = bfReader.readLine()) != null) {
                    builder.append(tmpContent);
                }
                if(StringUtils.isNotBlank(builder.toString())){
                    retStr = builder.toString().replaceAll( "/\\*[\\s\\S]*?\\*/", "" );
                }
                String callbackName = request.getParameter("callback");
                if(StringUtils.isNotBlank(callbackName)){
                    retStr = callbackName+"("+retStr+");";
                    response.setContentType("application/javascript");
                }
                outputStream.write(retStr.getBytes());
            } else if ("uploadimage".equals(action)){
                String url = "errorUploadImage";
                JSONObject jsonObject = new JSONObject();
                if(upfile!=null){
                    String fix = FileNameUtil.getSuffix(upfile.getOriginalFilename());
                    String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
                    url = fileService.createFile(path, IoUtil.readBytes(upfile.getInputStream()));
                    jsonObject.put("type",StrUtil.DOT + fix);
                }
                jsonObject.put("state","SUCCESS");
                jsonObject.put("url",url);
                jsonObject.put("title",upfile.getOriginalFilename());
                jsonObject.put("original",upfile.getOriginalFilename());
                outputStream.write(jsonObject.toJSONString().getBytes());
            }
        } catch (Exception e) {
            e.printStackTrace();
            retStr = e.getMessage();
            outputStream.write(retStr.getBytes());
        } finally {
            try {
                if(bfReader!=null){
                    bfReader.close();
                }
                if(reader!=null){
                    reader.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
